/*
 * kaffeh-support.h
 *
 * Copyright (c) 1996 T. J. Wilkinson & Associates, London, UK.
 *
 * See the file "license.terms" for information on usage and redistribution
 * of this file, and for a DISCLAIMER OF ALL WARRANTIES.
 *
 * Written by Tim Wilkinson <tim@tjwassoc.demon.co.uk>, 1996.
 */

#ifndef __kaffeh_support_h
#define __kaffeh_support_h

void initStub(void);
void startStub(void);

void initInclude(void);
void startInclude(void);
void endInclude(void);

void findClass(char *nm);
 
#endif
